@@ -69,7 +69,7 @@ Atom::Atom (): gcu::Atom (),
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
 	m_ChargeAutoPos = true;
-	m_Layout = m_ChargeLayout = NULL;
+	m_Layout = m_ChargeLayout = m_HLayout = NULL;
 	m_DrawCircle = false;
 	m_SWidth = 0.;
 }
@@ -111,7 +111,7 @@ Atom::Atom (int Z, double x, double y, double z): gcu::Atom (Z, x, y, z),
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
 	m_ChargeAutoPos = true;
-	m_Layout = m_ChargeLayout = NULL;
+	m_Layout = m_ChargeLayout = m_HLayout = NULL;
 	m_DrawCircle = false;
 	m_SWidth = 0.;
 }
@@ -128,7 +128,7 @@ Atom::Atom (OBAtom* atom): gcu::Atom (),
 	gchar* Id = g_strdup_printf ("a%d", atom->GetIdx());
 	SetId (Id);
 	g_free (Id);
-	m_HPos = true;
+	m_HPos = GetBestSide ();
 	m_ascent = 0;
 	m_CHeight = 0.;
 	m_Changed = 0;
@@ -138,7 +138,7 @@ Atom::Atom (OBAtom* atom): gcu::Atom (),
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
 	m_ChargeAutoPos = true;
-	m_Layout = m_ChargeLayout = NULL;
+	m_Layout = m_ChargeLayout = m_HLayout = NULL;
 	m_DrawCircle = false;
 	m_Charge = atom->GetFormalCharge ();
 	m_SWidth = 0.;
@@ -205,18 +205,22 @@ void Atom::RemoveBond (gcu::Bond* pBond)
 	Update ();
 }
 
-bool Atom::GetBestSide ()
+HPos Atom::GetBestSide ()
 {
-	if (m_Bonds.size () == 0)
-		return Element::BestSide (m_Z);
+	size_t nb_bonds = m_Bonds.size ();
+	if (nb_bonds == 0)
+		return (Element::BestSide (m_Z))? RIGHT_HPOS: LEFT_HPOS;
 	std::map<gcu::Atom*, gcu::Bond*>::iterator i, end = m_Bonds.end();
-	double sum = 0.0;
-	for (i = m_Bonds.begin(); i != end; i++)
-		sum -= cos(((Bond*) (*i).second)->GetAngle2DRad (this));
-	if (fabs(sum) > 0.1)
-		return (sum >= 0.0);
+	double sumc = 0.0, sums = 0.0, a;
+	for (i = m_Bonds.begin(); i != end; i++) {
+		a = ((Bond*) (*i).second)->GetAngle2DRad (this);
+		sumc += cos (a);
+		sums += sin (a);
+	}
+	if (fabs (sums) > fabs (sumc) && nb_bonds > 1)
+		return (fabs (sums) > .1)? ((sums >= 0.)? BOTTOM_HPOS: TOP_HPOS): ((Element::BestSide (m_Z))? RIGHT_HPOS: LEFT_HPOS);
 	else
-		return Element::BestSide (m_Z);
+		return (fabs (sumc) > .1)? ((sumc >= 0.)? LEFT_HPOS: RIGHT_HPOS): ((Element::BestSide (m_Z))? RIGHT_HPOS: LEFT_HPOS);
 }
 
 void Atom::Update ()
@@ -283,27 +287,28 @@ void Atom::Update ()
 	if (pDoc && pDoc->GetView ())
 		m_Changed = pDoc->GetView ()->GetNbWidgets ();
 	m_AvailPosCached = false;
+	map<gcu::Atom*, gcu::Bond*>::iterator j = m_Bonds.begin(), jend = m_Bonds.end ();
 	if (nbonds && GetZ () == 6) {
 		// update large bonds ends
 		Bond *bond;
 		BondType type;
 		bool DrawCircle;
-		map<gcu::Atom*, gcu::Bond*>::iterator i = m_Bonds.begin(), iend = m_Bonds.end ();
 		int nb = 0;
-		while (i != iend)
+		j = m_Bonds.begin();
+		while (j != jend)
 		{
-			bond = dynamic_cast<Bond*> ((Bond*)(*i).second);
+			bond = dynamic_cast<Bond*> ((Bond*)(*j).second);
 			type = bond->GetType ();
 			if (type == ForeBondType || (type == UpBondType && bond->GetAtom (1) == this))
 				nb++;
-			i++;
+			j++;
 		}
 		DrawCircle = nb > 1;
 		if (!DrawCircle && GetBondsNumber () == 2) {
-			i = m_Bonds.begin();
-			double angle = static_cast<Bond*> ((*i).second)->GetAngle2D (this);
-			i++;
-			angle -= static_cast<Bond*> ((*i).second)->GetAngle2D (this);
+			j = m_Bonds.begin();
+			double angle = static_cast<Bond*> ((*j).second)->GetAngle2D (this);
+			j++;
+			angle -= static_cast<Bond*> ((*j).second)->GetAngle2D (this);
 			if (go_finite (angle)) {
 				while (angle < 0)
 					angle += 360.;
@@ -318,6 +323,10 @@ void Atom::Update ()
 			m_Changed = true;
 		}
 	}
+	// Update all double bonds
+	for (j = m_Bonds.begin(); j != jend; j++)
+		if (((*j).second)->GetOrder () == 2)
+			static_cast<Bond*> ((*j).second)->SetDirty ();
 }
 
 void Atom::Add (GtkWidget* w) const
@@ -334,6 +343,7 @@ void Atom::Add (GtkWidget* w) const
 	if (m_Layout == NULL) {
 		PangoContext* pc = pView->GetPangoContext ();
 		const_cast <Atom *> (this)->m_Layout = pango_layout_new (pc);
+		const_cast <Atom *> (this)->m_HLayout = pango_layout_new (pc);
 	}
 	if (m_FontName != pView->GetFontName ()) {
 		pango_layout_set_font_description (m_Layout, pView->GetPangoFontDesc ());
@@ -361,74 +371,96 @@ void Atom::Add (GtkWidget* w) const
 	g_signal_connect (G_OBJECT (group), "event", G_CALLBACK (on_event), w);
 	g_object_set_data (G_OBJECT (group), "object", (void *) this);
 	if ((GetZ () != 6) || (GetBondsNumber () == 0)) {
-		int sw, sp;
+		int sw;
 		const gchar* symbol = GetSymbol (), *text;
 		sw = strlen (symbol);
 		pango_layout_set_text (m_Layout, symbol, sw);
 		pango_layout_get_extents (m_Layout, &rect, NULL);
 		const_cast <Atom *> (this)->m_width += rect.width / PANGO_SCALE;
 		const_cast <Atom *> (this)->BuildSymbolGeometry ((double) rect.width / PANGO_SCALE, (double) rect.height / PANGO_SCALE, m_ascent - (double) rect.y / PANGO_SCALE - m_CHeight);
-/*			m_SWidth = (double) rect.width / PANGO_SCALE / 2.;
-		const_cast <Atom *> (this)->m_SHeightH = m_ascent - (double) rect.y / PANGO_SCALE - m_CHeight;
-		const_cast <Atom *> (this)->m_SHeightL = (double) rect.height / PANGO_SCALE - m_SHeightH;
-		const_cast <Atom *> (this)->m_SAngleH = atan2 (m_SHeightH, m_SWidth);
-		const_cast <Atom *> (this)->m_SAngleL = atan2 (m_SHeightL, m_SWidth);*/
-		int n = GetAttachedHydrogens ();
-		if (n > 0) {
-			if (n > 1) {
-				gchar const *nb =  g_strdup_printf ("%d", n);
-				int np, nw = strlen (nb);
-				if (m_HPos) {
-					text = g_strconcat (symbol, "H", nb, NULL);
-					np = sw + 1;
-					sp = 0;
-				} else {
-					text = g_strconcat ("H", nb, symbol, NULL);
-					np = 1;
-					sp = np + nw;
-				}
-				pango_layout_set_text (m_Layout, text, -1);
-				PangoAttrList *pal = pango_attr_list_new ();
-				PangoAttribute *attr = pango_attr_font_desc_new (pView->GetPangoSmallFontDesc());
-				attr->start_index = np;
-				attr->end_index = np + nw;
-				pango_attr_list_insert (pal, attr);
-				attr = pango_attr_rise_new (-2 * PANGO_SCALE);
-				attr->start_index = np;
-				attr->end_index = np + nw;
-				pango_attr_list_insert (pal, attr);
-				pango_layout_set_attributes (m_Layout, pal);
-				pango_attr_list_unref (pal);
-			} else {
-				if (m_HPos) {
-					text = g_strconcat (symbol, "H", NULL);
-					sp = 0;
-				} else {
-					text = g_strconcat ("H", symbol, NULL);
-					sp = 1;
-				}
-				pango_layout_set_text (m_Layout, text, -1);
-			}
-		} else {
-			text = g_strdup (symbol);
-			sp = 0;
-			pango_layout_set_text (m_Layout, text, -1);
-		}
+		const_cast <Atom *> (this)->m_lbearing = m_width / 2.;
+		const_cast <Atom *> (this)->m_xROffs = const_cast <Atom *> (this)->m_yROffs = 0.;
 		pango_layout_get_extents (m_Layout, NULL, &rect);
 		const_cast <Atom *> (this)->m_length = double (rect.width / PANGO_SCALE);
 		const_cast <Atom *> (this)->m_text_height = const_cast <Atom *> (this)->m_height = rect.height / PANGO_SCALE;
-		pango_layout_index_to_pos (m_Layout, sp, &rect);
-		int st = rect.x / PANGO_SCALE;
-		pango_layout_index_to_pos (m_Layout, sp + sw, &rect);
-		const_cast <Atom *> (this)->m_lbearing = (st + rect.x / PANGO_SCALE) / 2.;
-
+		int n = GetAttachedHydrogens ();
+		if (n > 0) {
+			pango_layout_set_text (m_HLayout, "H", -1);
+			PangoRectangle HRect;
+			HRect.width = 0.;
+			pango_layout_get_extents (m_HLayout, &HRect, NULL);
+			switch (m_HPos) {
+			case LEFT_HPOS:
+				//the x offset needs to be calculated after adding possible stoichimoetry.
+				const_cast <Atom *> (this)->m_yHOffs = 0.;
+				break;
+			case RIGHT_HPOS:
+				const_cast <Atom *> (this)->m_xHOffs = rect.width / PANGO_SCALE + 1.;
+				const_cast <Atom *> (this)->m_yHOffs = 0.;
+				break;
+			case TOP_HPOS:
+				const_cast <Atom *> (this)->m_xHOffs = m_lbearing - pTheme->GetPadding () - HRect.width / PANGO_SCALE / 2.;
+				break;
+			case BOTTOM_HPOS:
+				const_cast <Atom *> (this)->m_xHOffs = m_lbearing - pTheme->GetPadding () - HRect.width / PANGO_SCALE / 2.;
+				const_cast <Atom *> (this)->m_yHOffs = m_CHeight * 2. + pTheme->GetPadding ();
+				break;
+			default:
+				g_critical ("This should not happen, please file a bug report");
+				break;
+			}
+			if (n > 1) {
+				gchar const *nb =  g_strdup_printf ("%d", n);
+				int nw = strlen (nb) + 1;
+				PangoAttrList *pal = pango_attr_list_new ();
+				text = g_strconcat ("H", nb, NULL);
+				pango_layout_set_text (m_HLayout, text, -1);
+				nw = strlen (text);
+				PangoAttribute *attr = pango_attr_font_desc_new (pView->GetPangoSmallFontDesc());
+				attr->start_index = 1;
+				attr->end_index = nw;
+				pango_attr_list_insert (pal, attr);
+				attr = pango_attr_rise_new (-2 * PANGO_SCALE);
+				attr->start_index = 1;
+				attr->end_index = nw;
+				pango_attr_list_insert (pal, attr);
+				pango_layout_set_attributes (m_HLayout, pal);
+				pango_attr_list_unref (pal);
+			}
+			pango_layout_get_extents (m_HLayout, &HRect, NULL);
+			// evaluate underlying rectangle size and position
+			if (HRect.width > 0) {
+				switch (m_HPos) {
+				case LEFT_HPOS:
+					const_cast <Atom *> (this)->m_xHOffs = -HRect.width / PANGO_SCALE - 1.;
+					const_cast <Atom *> (this)->m_xROffs = m_xHOffs;
+					const_cast <Atom *> (this)->m_length += HRect.width / PANGO_SCALE + 1.;
+					break;
+				case RIGHT_HPOS:
+					const_cast <Atom *> (this)->m_length += HRect.width / PANGO_SCALE + 1.;
+					break;
+				case TOP_HPOS:
+					const_cast <Atom *> (this)->m_yHOffs = -HRect.height / PANGO_SCALE - pTheme->GetPadding ();
+					const_cast <Atom *> (this)->m_yROffs = -HRect.height / PANGO_SCALE - pTheme->GetPadding ();
+					const_cast <Atom *> (this)->m_text_height += HRect.height / PANGO_SCALE + pTheme->GetPadding ();
+					const_cast <Atom *> (this)->m_length = MAX (m_length, HRect.width / PANGO_SCALE);
+					break;
+				case BOTTOM_HPOS:
+					const_cast <Atom *> (this)->m_text_height += HRect.height / PANGO_SCALE + pTheme->GetPadding ();
+					const_cast <Atom *> (this)->m_length = MAX (m_length, HRect.width / PANGO_SCALE);
+					break;
+				default:
+					break;
+				}
+			}
+		}
 		item = gnome_canvas_item_new (
 							group,
 							gnome_canvas_rect_ext_get_type (),
-							"x1", x - m_lbearing - pTheme->GetPadding (),
-							"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding (),
-							"x2", x - m_lbearing + m_length + pTheme->GetPadding (),
-							"y2", y  - m_ascent + m_CHeight + m_height + pTheme->GetPadding (),
+							"x1", x - m_lbearing + m_xROffs,
+							"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding () + m_yROffs,
+							"x2", x - m_lbearing + m_length + 2 * pTheme->GetPadding () + m_xROffs,
+							"y2", y  - m_ascent + m_CHeight + m_text_height + pTheme->GetPadding () + m_yROffs,
 							"fill_color", (pData->IsSelected (this))? SelectColor: NULL,
 							NULL);
 		g_object_set_data (G_OBJECT (group), "rect", item);
@@ -446,6 +478,16 @@ void Atom::Add (GtkWidget* w) const
 		g_object_set_data (G_OBJECT (group), "symbol", item);
 		g_object_set_data (G_OBJECT (item), "object", (void *) this);
 		g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
+		item = gnome_canvas_item_new (
+							group,
+							gnome_canvas_pango_get_type (),
+							"x", x - m_lbearing + m_xHOffs,
+							"y", y - m_ascent + m_CHeight + m_yHOffs,
+							"layout", m_HLayout,
+						NULL);
+		g_object_set_data (G_OBJECT (group), "hydrogens", item);
+		g_object_set_data (G_OBJECT (item), "object", (void *) this);
+		g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
 	} else {
 		item = gnome_canvas_item_new (
 								group,
@@ -634,11 +676,16 @@ void Atom::Update (GtkWidget* w) const
 								"x", x - m_lbearing,
 								"y", y - m_ascent + m_CHeight,
 								NULL);
+			if (GetAttachedHydrogens ())
+				g_object_set (G_OBJECT (g_object_get_data (G_OBJECT (group), "hydrogens")),
+									"x", x - m_lbearing + m_xHOffs,
+									"y", y - m_ascent + m_CHeight + m_yHOffs,
+									NULL);
 			g_object_set (G_OBJECT (g_object_get_data (G_OBJECT (group), "rect")),
-								"x1", x - m_lbearing - pTheme->GetPadding (),
-								"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding (),
-								"x2", x - m_lbearing + m_length + pTheme->GetPadding (),
-								"y2", y  - m_ascent + m_CHeight + m_text_height + pTheme->GetPadding (),
+								"x1", x - m_lbearing + m_xROffs,
+								"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding () + m_yROffs,
+								"x2", x - m_lbearing + m_length + 2 * pTheme->GetPadding () + m_xROffs,
+								"y2", y  - m_ascent + m_CHeight + m_text_height + pTheme->GetPadding () + m_yROffs,
 								NULL);
 		} else {
 			g_object_set (G_OBJECT (g_object_get_data (G_OBJECT (group), "rect")),
@@ -885,14 +932,31 @@ void Atom::UpdateAvailablePositions ()
 	double angle, delta, dir;
 	m_AngleList.clear ();
 	if (((GetZ() != 6 || m_Bonds.size() == 0)) && m_nH) {
-		if (m_HPos) {
-			m_AvailPos = 0xB6;
-			m_AngleList.push_front(315.0);
-			m_AngleList.push_front(45.0);
-		} else {
+		switch (m_HPos) {
+		case LEFT_HPOS:
 			m_AvailPos = 0x6D;
 			m_AngleList.push_front(225.0);
 			m_AngleList.push_front(135.0);
+			break;
+		case RIGHT_HPOS:
+			m_AvailPos = 0xB6;
+			m_AngleList.push_front(315.0);
+			m_AngleList.push_front(45.0);
+			break;
+		case TOP_HPOS:
+			m_AvailPos = 0xF8;
+			m_AngleList.push_front(135.0);
+			m_AngleList.push_front(45.0);
+			break;
+			break;
+		case BOTTOM_HPOS:
+			m_AvailPos = 0xC7;
+			m_AngleList.push_front(315.0);
+			m_AngleList.push_front(225.0);
+			break;
+			break;
+		default:
+			break;
 		}
 	} else
 		m_AvailPos = 0xff;
@@ -906,30 +970,30 @@ void Atom::UpdateAvailablePositions ()
 		while ((n != m_AngleList.end ()) && (*n < angle)) n++;
 		m_AngleList.insert (n, angle);
 		i++;
-		if ((m_AvailPos & CHARGE_SW) && (angle >= 180.0 - ATOM_EPSILON) &&
+		if ((m_AvailPos & POSITION_SW) && (angle >= 180.0 - ATOM_EPSILON) &&
 			(angle <= 270.0 + ATOM_EPSILON))
-			m_AvailPos -= CHARGE_SW;
-		if ((m_AvailPos & CHARGE_SE) && (((angle >= 270.0 - ATOM_EPSILON) &&
+			m_AvailPos -= POSITION_SW;
+		if ((m_AvailPos & POSITION_SE) && (((angle >= 270.0 - ATOM_EPSILON) &&
 			(angle <= 360.0 + ATOM_EPSILON)) || (fabs(angle) < ATOM_EPSILON)))
-			m_AvailPos -= CHARGE_SE;
-		if ((m_AvailPos & CHARGE_S) && (angle >= 225.0 - ATOM_EPSILON) &&
+			m_AvailPos -= POSITION_SE;
+		if ((m_AvailPos & POSITION_S) && (angle >= 225.0 - ATOM_EPSILON) &&
 			(angle <= 315.0 + ATOM_EPSILON))
-			m_AvailPos -= CHARGE_S;
-		if ((m_AvailPos & CHARGE_NW) && (angle >= 90.0 - ATOM_EPSILON) &&
+			m_AvailPos -= POSITION_S;
+		if ((m_AvailPos & POSITION_NW) && (angle >= 90.0 - ATOM_EPSILON) &&
 			(angle <= 180.0 + ATOM_EPSILON))
-			m_AvailPos -= CHARGE_NW;
-		if ((m_AvailPos & CHARGE_NE) && (((angle >= 0.0 - ATOM_EPSILON) &&
+			m_AvailPos -= POSITION_NW;
+		if ((m_AvailPos & POSITION_NE) && (((angle >= 0.0 - ATOM_EPSILON) &&
 			(angle <= 90.0 + ATOM_EPSILON)) || (fabs(angle - 360.0) < ATOM_EPSILON)))
-			m_AvailPos -= CHARGE_NE;
-		if ((m_AvailPos & CHARGE_N) && (angle >= 45.0 - ATOM_EPSILON) &&
+			m_AvailPos -= POSITION_NE;
+		if ((m_AvailPos & POSITION_N) && (angle >= 45.0 - ATOM_EPSILON) &&
 			(angle <= 135.0 + ATOM_EPSILON))
-			m_AvailPos -= CHARGE_N;
-		if ((m_AvailPos & CHARGE_W) && ((angle <= 225.0 + ATOM_EPSILON) &&
+			m_AvailPos -= POSITION_N;
+		if ((m_AvailPos & POSITION_W) && ((angle <= 225.0 + ATOM_EPSILON) &&
 			(angle >= 135.0 - ATOM_EPSILON)))
-			m_AvailPos -= CHARGE_W;
-		if ((m_AvailPos & CHARGE_E) && ((angle >= 315.0 - ATOM_EPSILON) ||
+			m_AvailPos -= POSITION_W;
+		if ((m_AvailPos & POSITION_E) && ((angle >= 315.0 - ATOM_EPSILON) ||
 			(angle <= 45.0 + ATOM_EPSILON)))
-			m_AvailPos -= CHARGE_E;
+			m_AvailPos -= POSITION_E;
 	}
 	m_AngleList.push_back ((angle = m_AngleList.front ()) + 360.0);
 	m_InterBonds.clear ();
@@ -962,22 +1026,22 @@ int Atom::GetChargePosition (unsigned char& Pos, double Angle, double& x, double
 			Angle = m_ChargeAngle * 180 / M_PI;
 	} else if (Pos == 0xff) {
 		if (m_AvailPos) {
-			if (m_AvailPos & CHARGE_NE)
-				Pos = CHARGE_NE;
-			else if (m_AvailPos & CHARGE_NW)
-				Pos = CHARGE_NW;
-			else if (m_AvailPos & CHARGE_N)
-				Pos = CHARGE_N;
-			else if (m_AvailPos & CHARGE_SE)
-				Pos = CHARGE_SE;
-			else if (m_AvailPos & CHARGE_SW)
-				Pos = CHARGE_SW;
-			else if (m_AvailPos & CHARGE_S)
-				Pos = CHARGE_S;
-			else if (m_AvailPos & CHARGE_E)
-				Pos = CHARGE_E;
-			else if (m_AvailPos & CHARGE_W)
-				Pos = CHARGE_W;
+			if (m_AvailPos & POSITION_NE)
+				Pos = POSITION_NE;
+			else if (m_AvailPos & POSITION_NW)
+				Pos = POSITION_NW;
+			else if (m_AvailPos & POSITION_N)
+				Pos = POSITION_N;
+			else if (m_AvailPos & POSITION_SE)
+				Pos = POSITION_SE;
+			else if (m_AvailPos & POSITION_SW)
+				Pos = POSITION_SW;
+			else if (m_AvailPos & POSITION_S)
+				Pos = POSITION_S;
+			else if (m_AvailPos & POSITION_E)
+				Pos = POSITION_E;
+			else if (m_AvailPos & POSITION_W)
+				Pos = POSITION_W;
 		} else {
 			Pos = 0;
 			angle = m_AngleList.front();
@@ -1013,36 +1077,36 @@ int Atom::GetChargePosition (unsigned char& Pos, double Angle, double& x, double
 			return 0;
 	}
 	switch (Pos) {
-	case CHARGE_NE:
+	case POSITION_NE:
 		x = m_x + m_width / 2.0;
 		y = m_y - m_height / 2.0;
 		return 1;
-	case CHARGE_NW:
+	case POSITION_NW:
 		x = m_x - m_width / 2.0;
 		y = m_y - m_height / 2.0;
 		return -1;
-	case CHARGE_N:
+	case POSITION_N:
 		x = m_x;
 		y = m_y - m_height / 2.0;
 		return 2;
-	case CHARGE_SE:
+	case POSITION_SE:
 		x = m_x + m_width / 2.0;
 		y = m_y + m_height / 2.0;
 		return 1;
-	case CHARGE_SW:
+	case POSITION_SW:
 		x = m_x - m_width / 2.0;
 		y = m_y + m_height / 2.0;
 		return -1;
-	case CHARGE_S:
+	case POSITION_S:
 		x = m_x;
 		y = m_y + m_height / 2.0;
 		return -2;
-	case CHARGE_E:
-		x = m_x /*+ 12.*/ + m_width / 2.0;
+	case POSITION_E:
+		x = m_x + m_width / 2.0;
 		y = m_y;
 		return 1;
-	case CHARGE_W:
-		x = m_x /*- 12.*/ - m_width / 2.0;
+	case POSITION_W:
+		x = m_x - m_width / 2.0;
 		y = m_y;
 		return -1;
 	default: {
@@ -1240,69 +1304,95 @@ void Atom::BuildItems (WidgetData* pData)
 	GetCoords (&x, &y);
 	x *= pTheme->GetZoomFactor ();
 	y *= pTheme->GetZoomFactor ();
+	m_xROffs = m_yROffs = 0.;
 	if ((GetZ() != 6) || (GetBondsNumber() == 0) || m_ShowSymbol) {
-		int sw, sp;
+		int sw;
 		const gchar* symbol = GetSymbol (), *text;
-		PangoRectangle rect;
+		PangoRectangle rect, HRect;
 		sw = strlen (symbol);
 		pango_layout_set_text (m_Layout, symbol, sw);
 		pango_layout_get_extents (m_Layout, &rect, NULL);
 		m_width += rect.width / PANGO_SCALE;
-		int n = GetAttachedHydrogens ();
-		PangoAttrList *pal = pango_attr_list_new ();
-		if (n > 0) {
-			if (n > 1) {
-				gchar const *nb =  g_strdup_printf ("%d", n);
-				int np, nw = strlen (nb);
-				if (m_HPos) {
-					text = g_strconcat (symbol, "H", nb, NULL);
-					np = sw + 1;
-					sp = 0;
-				} else {
-					text = g_strconcat ("H", nb, symbol, NULL);
-					np = 1;
-					sp = np + nw;
-				}
-				pango_layout_set_text (m_Layout, text, -1);
-				PangoAttribute *attr = pango_attr_font_desc_new (pView->GetPangoSmallFontDesc());
-				attr->start_index = np;
-				attr->end_index = np + nw;
-				pango_attr_list_insert (pal, attr);
-				attr = pango_attr_rise_new (-2 * PANGO_SCALE);
-				attr->start_index = np;
-				attr->end_index = np + nw;
-				pango_attr_list_insert (pal, attr);
-			} else {
-				if (m_HPos) {
-					text = g_strconcat (symbol, "H", NULL);
-					sp = 0;
-				} else {
-					text = g_strconcat ("H", symbol, NULL);
-					sp = 1;
-				}
-				pango_layout_set_text (m_Layout, text, -1);
-			}
-			pango_layout_set_attributes (m_Layout, pal);
-			pango_attr_list_unref (pal);
-		} else {
-			text = g_strdup (symbol);
-			sp = 0;
-			pango_layout_set_text (m_Layout, text, -1);
-		}
 		pango_layout_get_extents (m_Layout, NULL, &rect);
 		m_length = double (rect.width / PANGO_SCALE);
 		m_text_height = m_height = rect.height / PANGO_SCALE;
-		pango_layout_index_to_pos (m_Layout, sp, &rect);
-		int st = rect.x / PANGO_SCALE;
-		pango_layout_index_to_pos (m_Layout, sp + sw, &rect);
-		m_lbearing = (st + rect.x / PANGO_SCALE) / 2.;
-
+		m_lbearing = m_width / 2.;
+		HRect.width = 0;
+		int n = GetAttachedHydrogens ();
+		if (n > 0) {
+			pango_layout_set_text (m_HLayout, "H", -1);
+			pango_layout_get_extents (m_HLayout, &HRect, NULL);
+			switch (m_HPos) {
+			case LEFT_HPOS:
+				//the x offset needs to be calculated after adding possible stoichimoetry.
+				m_xHOffs = m_yHOffs = 0.;
+				break;
+			case RIGHT_HPOS:
+				m_xHOffs = rect.width / PANGO_SCALE + 1.;
+				m_yHOffs = 0.;
+				break;
+			case TOP_HPOS:
+				m_xHOffs = m_lbearing - pTheme->GetPadding () - HRect.width / PANGO_SCALE / 2.;
+				break;
+			case BOTTOM_HPOS:
+				m_xHOffs = m_lbearing - pTheme->GetPadding () - HRect.width / PANGO_SCALE / 2.;
+				m_yHOffs = m_CHeight * 2. + pTheme->GetPadding ();
+				break;
+			default:
+				g_critical ("This should not happen, please file a bug report");
+				break;
+			}
+			if (n > 1) {
+				gchar const *nb =  g_strdup_printf ("%d", n);
+				int nw = strlen (nb);
+				PangoAttrList *pal = pango_attr_list_new ();
+				text = g_strconcat ("H", nb, NULL);
+				pango_layout_set_text (m_HLayout, text, -1);
+				nw = strlen (text);
+				PangoAttribute *attr = pango_attr_font_desc_new (pView->GetPangoSmallFontDesc());
+				attr->start_index = 1;
+				attr->end_index = strlen (text);
+				pango_attr_list_insert (pal, attr);
+				attr = pango_attr_rise_new (-2 * PANGO_SCALE);
+				attr->start_index = 1;
+				attr->end_index = nw;
+				pango_attr_list_insert (pal, attr);
+				pango_layout_set_attributes (m_HLayout, pal);
+				pango_attr_list_unref (pal);
+			}
+			pango_layout_get_extents (m_HLayout, &HRect, NULL);
+			// evaluate underlying rectangle size and position
+			if (HRect.width > 0) {
+				switch (m_HPos) {
+				case LEFT_HPOS:
+					m_xHOffs = -HRect.width / PANGO_SCALE - 1.;
+					m_xROffs = m_xHOffs;
+					m_length += HRect.width / PANGO_SCALE + 1.;
+					break;
+				case RIGHT_HPOS:
+					m_length += HRect.width / PANGO_SCALE + 1.;
+					break;
+				case TOP_HPOS:
+					m_yHOffs = -HRect.height / PANGO_SCALE - pTheme->GetPadding ();
+					m_yROffs = -HRect.height / PANGO_SCALE - pTheme->GetPadding ();
+					m_text_height += HRect.height / PANGO_SCALE + pTheme->GetPadding ();
+					m_length = MAX (m_length, HRect.width / PANGO_SCALE);
+					break;
+				case BOTTOM_HPOS:
+					m_text_height += HRect.height / PANGO_SCALE + pTheme->GetPadding ();
+					m_length = MAX (m_length, HRect.width / PANGO_SCALE);
+					break;
+				default:
+					break;
+				}
+			}
+		}
 		item = g_object_get_data (G_OBJECT (group), "rect");
 		g_object_set (item,
-							"x1", x - m_lbearing - pTheme->GetPadding (),
-							"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding (),
-							"x2", x - m_lbearing + m_length + pTheme->GetPadding (),
-							"y2", y  - m_ascent + m_CHeight + m_height + pTheme->GetPadding (),
+							"x1", x - m_lbearing + m_xROffs,
+							"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding () + m_yROffs,
+							"x2", x - m_lbearing + m_length + 2 * pTheme->GetPadding () + m_xROffs,
+							"y2", y  - m_ascent + m_CHeight + m_text_height + pTheme->GetPadding () + m_yROffs,
 							NULL);
 		
 		item = g_object_get_data (G_OBJECT (group), "symbol");
@@ -1324,6 +1414,29 @@ void Atom::BuildItems (WidgetData* pData)
 			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), pData->Canvas);
 			gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
 		}
+		item = g_object_get_data (G_OBJECT (group), "hydrogens");
+		if (item) {
+			if (n == 0) {
+				gtk_object_destroy (GTK_OBJECT (item));
+				g_object_set_data ((GObject*)group, "hydrogens", NULL);
+			} else
+				g_object_set (item,
+								"x", x - m_lbearing + m_xHOffs,
+								"y", y - m_ascent + m_CHeight + m_yHOffs,
+								NULL);
+		} else if (n > 0) {
+			item = gnome_canvas_item_new (
+							group,
+							gnome_canvas_pango_get_type(),
+							"layout", m_HLayout,
+							"x", x - m_lbearing + m_xHOffs,
+							"y", y - m_ascent + m_CHeight + m_yHOffs,
+							NULL);
+			g_object_set_data (G_OBJECT (group), "hydrogens", item);
+			g_object_set_data (G_OBJECT (item), "object", this);
+			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), pData->Canvas);
+			gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
+		}
 		item = g_object_get_data (G_OBJECT (group), "bullet");
 		if (item) {
 			gtk_object_destroy (GTK_OBJECT (item));
@@ -1342,6 +1455,11 @@ void Atom::BuildItems (WidgetData* pData)
 			gtk_object_destroy (GTK_OBJECT (item));
 			g_object_set_data (G_OBJECT (group), "symbol", NULL);
 		}
+		item = g_object_get_data (G_OBJECT (group), "hydrogens");
+		if (item) {
+			gtk_object_destroy (GTK_OBJECT (item));
+			g_object_set_data ((GObject*)group, "hydrogens", NULL);
+		}
 		item = g_object_get_data (G_OBJECT (group), "bullet");
 		if (m_DrawCircle) {
 			if (!item) {
@@ -1383,7 +1501,7 @@ bool Atom::HasImplicitElectronPairs ()
 	Electron* electron = (Electron*) GetFirstChild (i);
 	if (m_Valence > 0) {
 		int nexplp = 0; //nexplp is the number of explicit lone pairs
-		while (electron){ 
+		while (electron) { 
 			if (electron->IsPair ())
 				nexplp++;
 			electron = (Electron*) GetNextChild (i);
@@ -1408,7 +1526,7 @@ bool Atom::MayHaveImplicitUnpairedElectrons ()
 	map<string, Object*>::iterator i;
 	Electron* electron = (Electron*) GetFirstChild (i);
 	unsigned nel = 0;
-	while (electron){ 
+	while (electron) { 
 		if (electron->IsPair ())
 			nel += 2;
 		else
@@ -1427,8 +1545,10 @@ bool Atom::GetPosition(double angle, double& x, double& y)
 	else if (angle < 0.)
 		angle += 360;
 	if (((GetZ() == 6) && (m_Bonds.size() != 0)) ||
-		 !m_nH || ((!m_HPos && (angle < 135. || angle > 225.)) ||
-			(m_HPos && (angle > 45. && angle < 315.)))) {
+		 !m_nH || ((m_HPos == LEFT_HPOS && (angle < 135. || angle > 225.)) ||
+			(m_HPos == RIGHT_HPOS && (angle > 45. && angle < 315.)) ||
+			(m_HPos == TOP_HPOS && (angle > 45. || angle > 135.)) ||
+			(m_HPos == BOTTOM_HPOS && (angle < 225. || angle > 315.)))) {
 		double t = tan (angle / 180. * M_PI);
 		double limit = atan (m_height / m_width) * 180. / M_PI;
 		if (angle < limit) {
@@ -1503,28 +1623,28 @@ xmlNodePtr Atom::Save (xmlDocPtr xml) const
 		if (m_ChargePos) {
 			char const *buf;
 			switch (m_ChargePos) {
-			case CHARGE_NE:
+			case POSITION_NE:
 				buf = "ne";
 				break;
-			case CHARGE_NW:
+			case POSITION_NW:
 				buf = "nw";
 				break;
-			case CHARGE_N:
+			case POSITION_N:
 				buf = "n";
 				break;
-			case CHARGE_SE:
+			case POSITION_SE:
 				buf = "se";
 				break;
-			case CHARGE_SW:
+			case POSITION_SW:
 				buf = "sw";
 				break;
-			case CHARGE_S:
+			case POSITION_S:
 				buf = "s";
 				break;
-			case CHARGE_E:
+			case POSITION_E:
 				buf = "e";
 				break;
-			case CHARGE_W:
+			case POSITION_W:
 				buf = "w";
 				break;
 			default:
@@ -1573,28 +1693,28 @@ bool Atom::Load (xmlNodePtr node)
 	m_ChargePos = 0xff;
 	if (buf) {
 		if (! strcmp (buf, "ne")) {
-			m_ChargePos = CHARGE_NE;
+			m_ChargePos = POSITION_NE;
 			m_ChargeAngle = M_PI / 4.;
 		} else if (! strcmp (buf, "nw")) {
-			m_ChargePos = CHARGE_NW;
+			m_ChargePos = POSITION_NW;
 			m_ChargeAngle = 3. * M_PI / 4.;
 		} else if (! strcmp (buf, "n")) {
-			m_ChargePos = CHARGE_N;
+			m_ChargePos = POSITION_N;
 			m_ChargeAngle = M_PI / 2.;
 		} else if (! strcmp (buf, "se")) {
-			m_ChargePos = CHARGE_SE;
+			m_ChargePos = POSITION_SE;
 			m_ChargeAngle = 7. * M_PI / 4;
 		} else if (! strcmp (buf, "sw")) {
-			m_ChargePos = CHARGE_SW;
+			m_ChargePos = POSITION_SW;
 			m_ChargeAngle = 5. * M_PI / 4;
 		} else if (! strcmp (buf, "s")) {
-			m_ChargePos = CHARGE_S;
+			m_ChargePos = POSITION_S;
 			m_ChargeAngle = 3 * M_PI / 2.;
 		} else if (! strcmp (buf, "e")) {
-			m_ChargePos = CHARGE_E;
+			m_ChargePos = POSITION_E;
 			m_ChargeAngle = 0.;
 		} else if (! strcmp (buf, "w")) {
-			m_ChargePos = CHARGE_W;
+			m_ChargePos = POSITION_W;
 			m_ChargeAngle = M_PI;
 		}
 		m_ChargeAutoPos = false;
