@@ -140,11 +140,12 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 				*x2 += ay * dx / dy;
 			}
 		}
+		// always set the first line coords, even if changed later
+		m_coords[0] = *x1;
+		m_coords[1] = *y1;
+		m_coords[2] = *x2;
+		m_coords[3] = *y2;
 		if (m_order & 1) {
-			m_coords[0] = *x1;
-			m_coords[1] = *y1;
-			m_coords[2] = *x2;
-			m_coords[3] = *y2;
 			if (m_order == 3) {
 				m_coords[4] = *x1 - dy;
 				m_coords[5] = *y1 + dx;
@@ -156,41 +157,176 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 				m_coords[11] = *y2 - dx;
 			}
 		} else if ((m_order == 2) && IsCyclic ()) {
-			m_coords[0] = *x1;
-			m_coords[1] = *y1;
-			m_coords[2] = *x2;
-			m_coords[3] = *y2;
 			Cycle* pCycle;
+			double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1, a2;
 			if (IsCyclic() > 1) {
 				//Search prefered cycle
 				list<Cycle*>::iterator i = m_Cycles.begin (), end = m_Cycles.end ();
 				pCycle = *i;
-				for (; i != end; i++)
-					if (pCycle->IsBetterForBonds (*i))
+				pCycle->GetAngles2D (this, &a1, &a2);
+				bool trans = sin(a0 - a1) * sin (a0 - a2) <= 0; // we prefer a cycle with cis configuration
+				for (; i != end; i++) {
+					if (trans) {
+						(*i)->GetAngles2D (this, &a1, &a2);
+						if (sin(a0 - a1) * sin (a0 - a2) > 0) {
+							trans = false;
+							pCycle = *i;
+							continue;
+						}
+					}
+					if (pCycle->IsBetterForBonds (*i)) {
+						if (!trans) {
+							(*i)->GetAngles2D (this, &a1, &a2);
+							if (sin(a0 - a1) * sin (a0 - a2) <= 0)
+								continue;
+						}
 						pCycle = *i;
+					}
+				}
 			} else
 				pCycle = m_Cycles.front();
-			double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1, a2;
 			pCycle->GetAngles2D (this, &a1, &a2);
 			if (sin(a0 - a1) * sin (a0 - a2) > 0) {
 				double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
-				double tanb = fabs (tan ((M_PI - a0 + a1) / 2)), cosa = cos (a0), sina = sin (a0);
+				double tanb = 0., cosa = cos (a0), sina = sin (a0);
+				if (m_Begin->GetZ () == 6 && !reinterpret_cast <Atom*> (m_Begin)->GetShowSymbol ())
+					tanb = fabs (tan ((M_PI - a0 + a1) / 2));
 				m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
 				m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
-				tanb = fabs (tan ((a2 - a0) / 2));
+				tanb = (m_End->GetZ () == 6 && !reinterpret_cast <Atom*> (m_End)->GetShowSymbol ())? fabs (tan ((a2 - a0) / 2)): 0.;
 				m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
 				m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
-			} else {
-				m_coords[0] = *x1 - dy / 2;
-				m_coords[1] = *y1 + dx / 2;
-				m_coords[2] = *x2 - dy / 2;
-				m_coords[3] = *y2 + dx / 2;
-				m_coords[4] = *x1 + dy / 2;
-				m_coords[5] = *y1 - dx / 2;
-				m_coords[6] = *x2 + dy / 2;
-				m_coords[7] = *y2 - dx / 2;
-			}
+			} else goto general;
 		} else {
+general:
+			// search how many bonds have each atom
+			int n1 = m_Begin->GetBondsNumber () - 1, n2 = m_End->GetBondsNumber () - 1;
+			if (n1 == 1) {
+				// put the second line on the bond side if any
+				map <gcu::Atom*, gcu::Bond*>::iterator it;
+				Bond *bond = reinterpret_cast <Bond*> (m_Begin->GetFirstBond (it));
+				if (bond == this)
+					bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
+				double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)), a2, a;
+				if (fabs (fabs (a0 - a1) - M_PI) > 0.01) {
+					double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
+					double tanb = ((m_Begin->GetZ () == 6)? fabs (tan ((M_PI - a0 + a1) / 2)): 0.), cosa = cos (a0), sina = sin (a0);
+					m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
+					m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
+					tanb = 0.;
+					a2 = M_PI + a0;
+					if (a2 > 2 * M_PI)
+						a2 -= 2 * M_PI;
+					bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+					if (m_End->GetZ () == 6)
+						while (bond) {
+							if (bond != this) {
+								a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End)) - a0) / 2);
+								if (sign * a < sign * tanb)
+									tanb = a;
+								
+							}
+							bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
+						}
+					m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
+					m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
+					goto done;
+				}
+			} else if (n1 > 1 && n2 > 0) {
+				if (n2 == 1) {
+					map <gcu::Atom*, gcu::Bond*>::iterator it;
+					Bond *bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+					if (bond == this)
+						bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
+					double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End)), a2, a;
+					if (fabs (fabs (a0 - a1) - M_PI) > 0.01) {
+						double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
+						double tanb = 0., cosa = cos (a0), sina = sin (a0);
+						a2 = M_PI + a0;
+						if (a2 > 2 * M_PI)
+							a2 -= 2 * M_PI;
+						bond = reinterpret_cast <Bond*> (m_Begin->GetFirstBond (it));
+						if (m_Begin->GetZ () == 6)
+							while (bond) {
+								if (bond != this) {
+									a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)) - a2) / 2);
+									if (sign * a < sign * tanb)
+										tanb = a;
+									
+								}
+								bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
+							}
+						m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
+						m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
+						tanb = (m_End->GetZ () == 6)? fabs (tan ((a1 - a0) / 2)): 0.;
+						m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
+						m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
+					}
+				} else {
+					// use the side with more room.
+					double tana = 0., tanb = 0., tanc = 0., tand = 0.;
+					double a, a0 = GetAngle2DRad (reinterpret_cast <Atom*> (m_End)), a1 = GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin));
+					double cosa = cos (a0), sina = sin (a0);
+					map <gcu::Atom*, gcu::Bond*>::iterator it;
+					Bond *bond;
+					if (m_Begin->GetZ () == 6) {
+						bond = reinterpret_cast <Bond*> (m_Begin->GetFirstBond (it));
+						while (bond) {
+							if (bond != this) {
+								a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)) - a0) / 2);
+								if (a > tana)
+									tana = a;
+								if (a < tanb)
+									tanb = a;
+								
+							}
+							bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
+						}
+					}
+					if (m_End->GetZ () == 6) {
+						bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+						while (bond) {
+							if (bond != this) {
+								a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End)) - a1) / 2);
+								if (a > tanc)
+									tanc = a;
+								if (a < tand)
+									tand = a;
+								
+							}
+							bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
+						}
+					}
+					if (tana - tand > tanc - tanb) {
+						m_coords[4] = *x1 + BondDist * cosa * tanb + dy;
+						m_coords[5] = *y1 - dx - BondDist * sina * tanb;
+						m_coords[6] = *x2 + BondDist * cosa * tanc + dy;
+						m_coords[7] = *y2 - dx - BondDist * sina * tanc;
+					} else {
+						m_coords[4] = *x1 - BondDist * cosa * tana - dy;
+						m_coords[5] = *y1 + dx + BondDist * sina * tana;
+						m_coords[6] = *x2 - BondDist * cosa * tand - dy;
+						m_coords[7] = *y2 + dx + BondDist * sina * tand;
+					}
+				}
+				goto done;
+			} else if (n2 == 1) { // n1 is 0
+				map <gcu::Atom*, gcu::Bond*>::iterator it;
+				Bond *bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+				if (bond == this)
+					bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
+				double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End));
+				if (fabs (fabs (a0 - a1) - M_PI) > 0.01) {
+					double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
+					double tanb, cosa = cos (a0), sina = sin (a0);
+					m_coords[4] = *x1 - dy * sign;
+					m_coords[5] = *y1 + dx * sign;
+					tanb = (m_End->GetZ () == 6)? fabs (tan ((a1 - a0) / 2)): 0.;
+					m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
+					m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
+					goto done;
+				}
+			}
 			m_coords[0] = *x1 - dy / 2;
 			m_coords[1] = *y1 + dx / 2;
 			m_coords[2] = *x2 - dy / 2;
@@ -212,6 +348,7 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 		}
 		m_CoordsCalc = true;
 	}
+done:
 	Num--;
 	Num *= 4;
 	*x1 = m_coords[Num++];
