@@ -2,7 +2,7 @@
  * Gnome Chemisty Utils
  * chem-viewer.c 
  *
- * Copyright (C) 2005-2006
+ * Copyright (C) 2005-2008
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
  * published by the Free Software Foundation; either version 2 of the
@@ -23,16 +23,28 @@
 #include <gcu/element.h>
 #include <gcu/gtkchem3dviewer.h>
 #include <gcu/gtkcrystalviewer.h>
+#include <gcu/gtkspectrumviewer.h>
 #include <gcu/chem3ddoc.h>
+#include <gcp/application.h>
+#include <gcp/document.h>
+#include <gcp/theme.h>
+#include <gcp/view.h>
+#include <gcp/widgetdata.h>
+#include <goffice/goffice.h>
+#include <goffice/app/go-plugin.h>
+#include <goffice/app/go-plugin-loader-module.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkplug.h>
-#include <libgnomevfs/gnome-vfs.h>
+#ifdef GOFFICE_IS_0_6
+#	include <libgnomevfs/gnome-vfs.h>
+#endif
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <string>
 #include <map>
+#include <string>
 #include <cstring>
 
 using namespace std;
@@ -40,6 +52,24 @@ using namespace gcu;
 
 static bool loaded_radii = false;
 
+class MozPaintApp: public gcp::Application
+{
+public:
+	MozPaintApp ();
+	virtual ~MozPaintApp ();
+
+	void OnFileNew (char const *Theme = NULL) {}
+	GtkWindow* GetWindow() {return NULL;}
+};
+
+MozPaintApp::MozPaintApp (): gcp::Application ()
+{
+}
+
+MozPaintApp::~MozPaintApp ()
+{
+}
+
 class ChemComp
 {
 public:
@@ -56,7 +86,9 @@ private:
 	string Filename, MimeType;
 	GtkWidget *Plug, *Viewer;
 	GdkWindow *Parent;
+	gcp::Document *Doc;
 	map<string, string> Params;
+	MozPaintApp *gcpApp;
 };
 
 ChemComp::ChemComp (void* instance, string& mime_type)
@@ -65,6 +97,8 @@ ChemComp::ChemComp (void* instance, string& mime_type)
 	MimeType = mime_type;
 	Xid = 0;
 	Plug = NULL;
+	Doc = NULL;
+	gcpApp = NULL;
 }
 
 ChemComp::~ChemComp ()
@@ -96,6 +130,13 @@ void ChemComp::SetWindow (XID xid)
 			GDK_WINDOW_XID (Plug->window));
 		if (MimeType == "application/x-gcrystal")
 			Viewer = gtk_crystal_viewer_new (NULL);
+		else if (MimeType == "application/x-gchempaint") {
+			if (!gcpApp)
+				gcpApp = new MozPaintApp ();
+			Doc = new gcp::Document (gcpApp, true, NULL);
+			Viewer = Doc->GetView ()->CreateNewWidget ();
+		} else 	if (MimeType == "chemical/x-jcamp-dx")
+			Viewer = gtk_spectrum_viewer_new (NULL);
 		else
 			Viewer = gtk_chem3d_viewer_new (NULL);
 		gtk_container_add (GTK_CONTAINER (Plug), Viewer);
@@ -115,9 +156,30 @@ void ChemComp::SetFilename (string& filename)
 		if (!xml || !xml->children || strcmp ((char*) xml->children->name, "crystal"))
 			return;
 		gtk_crystal_viewer_set_data (GTK_CRYSTAL_VIEWER (Viewer), xml->children);
+		xmlFree (xml);
+	} else 	if (MimeType == "application/x-gchempaint") {
+		xmlDocPtr xml = xmlParseFile (filename.c_str ());
+		if (!xml || !xml->children || strcmp ((char*) xml->children->name, "chemistry"))
+			return;
+		Doc->Load (xml->children);
+		ArtDRect r;
+		gcp::WidgetData *pData = (gcp::WidgetData*) g_object_get_data (G_OBJECT (Viewer), "data");
+		pData->GetObjectBounds (Doc, &r);
+		gcp::Theme *pTheme = Doc->GetTheme ();
+		if (r.x0 || r.y0)
+			Doc->Move (- r.x0 / pTheme->GetZoomFactor (), - r.y0 / pTheme->GetZoomFactor ());
+		Doc->GetView ()->Update (Doc);
+		pData->GetObjectBounds (Doc, &r);
+		xmlFree (xml);
+	} else 	if (MimeType == "chemical/x-jcamp-dx") {
+		char *uri = g_filename_to_uri (filename.c_str (), NULL, NULL);
+		gtk_spectrum_viewer_set_uri (GTK_SPECTRUM_VIEWER (Viewer), uri);
+		g_free (uri);
 	} else {
+		char *uri = g_filename_to_uri (filename.c_str (), NULL, NULL);
 		gtk_chem3d_viewer_set_uri_with_mime_type (GTK_CHEM3D_VIEWER (Viewer),
-				filename.c_str (), MimeType.c_str ());
+				uri, MimeType.c_str ());
+		g_free (uri);
 		map<string, string>::iterator i, iend = Params.end ();
 		for (i = Params.begin (); i != iend; i++) {
 			if ((*i).first == "display3d") {
@@ -230,10 +292,14 @@ int main (int argc, char *argv[])
 	GError *error = NULL;
 
 	gtk_init (&argc, &argv);
+#ifdef GOFFICE_IS_0_6
 	if (!gnome_vfs_init ()) {
 		cerr << "Could not initialize GnomeVFS\n" << endl;
 		return 1;
 	}
+#endif
+	libgoffice_init ();
+	go_plugins_init (NULL, NULL, NULL, NULL, TRUE, GO_PLUGIN_LOADER_MODULE_TYPE);
 	in_channel = g_io_channel_unix_new (fileno (stdin));
 	g_io_add_watch (in_channel, G_IO_IN, io_func, &error);
 	gtk_main ();
